@@ -96,7 +96,7 @@ void Array::sinit(Class_base* c)
 	c->setMethodByQName("indexOf",AS3,Class<IFunction>::getFunction(indexOf),true);
 	c->setMethodByQName("lastIndexOf",AS3,Class<IFunction>::getFunction(lastIndexOf),true);
 	c->setMethodByQName("join",AS3,Class<IFunction>::getFunction(join),true);
-	//c->setMethodByQName("map",AS3,Class<IFunction>::getFunction(map),true);
+	c->setMethodByQName("map",AS3,Class<IFunction>::getFunction(_map),true);
 	c->setMethodByQName("pop",AS3,Class<IFunction>::getFunction(_pop),true);
 	c->setMethodByQName("push",AS3,Class<IFunction>::getFunction(_push),true);
 	c->setMethodByQName("reverse",AS3,Class<IFunction>::getFunction(_reverse),true);
@@ -239,7 +239,7 @@ ASFUNCTIONBODY(Array,lastIndexOf)
 
 	int unsigned i = th->data.size()-1;
 	int j;
-	if(argslen == 2 && args[1]->getObjectType() != T_UNDEFINED && !isnan(args[1]->toNumber()))
+	if(argslen == 2 && args[1]->getObjectType() != T_UNDEFINED && !std::isnan(args[1]->toNumber()))
 	{
 		j = args[1]->toInt(); //Preserve sign
 		if(j < 0) //Negative offset, use it as offset from the end of the array
@@ -357,7 +357,7 @@ ASFUNCTIONBODY(Array,_pop)
 {
 	Array* th=static_cast<Array*>(obj);
 	ASObject* ret;
-	if(th->data.back().type==DATA_OBJECT)
+	if(!th->data.empty() && th->data.back().type==DATA_OBJECT)
 		ret=th->data.back().data;
 	else
 		throw UnsupportedException("Array::pop not completely implemented");
@@ -381,7 +381,7 @@ bool Array::sortComparatorDefault::operator()(const data_slot& d1, const data_sl
 		else if(d2.type==DATA_OBJECT && d2.data)
 			b=d2.data->toNumber();
 
-		if(isnan(a) || isnan(b))
+		if(std::isnan(a) || std::isnan(b))
 			throw RunTimeException("Cannot sort non number with Array.NUMERIC option");
 		return a<b;
 	}
@@ -499,6 +499,38 @@ ASFUNCTIONBODY(Array,_push)
 	return abstract_i(th->size());
 }
 
+ASFUNCTIONBODY(Array,_map)
+{
+	Array* th=static_cast<Array*>(obj);
+	assert_and_throw(argslen==1 && args[0]->getObjectType()==T_FUNCTION);
+	IFunction* func=static_cast<IFunction*>(args[0]);
+	Array* arrayRet=Class<Array>::getInstanceS();
+
+	for(uint32_t i=0;i<th->data.size();i++)
+	{
+		ASObject* funcArgs[3];
+		const data_slot& slot=th->data[i];
+		if(slot.type==DATA_INT)
+			funcArgs[0]=abstract_i(slot.data_i);
+		else if(slot.type==DATA_OBJECT && slot.data)
+		{
+			funcArgs[0]=slot.data;
+			funcArgs[0]->incRef();
+		}
+		else
+			funcArgs[0]=new Undefined;
+
+		funcArgs[1]=abstract_i(i);
+		funcArgs[2]=th;
+		funcArgs[2]->incRef();
+		ASObject* funcRet=func->call(new Null, funcArgs, 3);
+		assert_and_throw(funcRet);
+		arrayRet->push(funcRet);
+	}
+
+	return arrayRet;
+}
+
 ASMovieClipLoader::ASMovieClipLoader()
 {
 }
@@ -1206,7 +1238,7 @@ TRISTATE ASString::isLess(ASObject* r)
 	}
 	number_t a=toNumber();
 	number_t b=r->toNumber();
-	if(isnan(a) || isnan(b))
+	if(std::isnan(a) || std::isnan(b))
 		return TUNDEFINED;
 	//TODO: Should we special handle infinite values?
 	return (a<b)?TTRUE:TFALSE;
@@ -2273,7 +2305,7 @@ ASFUNCTIONBODY(ASString,lastIndexOf)
 	ASString* th=static_cast<ASString*>(obj);
 	const tiny_string& val=args[0]->toString();
 	int startIndex=0x7fffffff;
-	if(argslen == 2 && args[1]->getObjectType() != T_UNDEFINED && !isnan(args[1]->toNumber()))
+	if(argslen == 2 && args[1]->getObjectType() != T_UNDEFINED && !std::isnan(args[1]->toNumber()))
 		startIndex=args[1]->toInt();
 		
 	if(startIndex < 0) //If negative offset is passed, clamp to 0 for start-of-string matches
@@ -2314,7 +2346,7 @@ ASFUNCTIONBODY(ASString,fromCharCode)
 
 ASFUNCTIONBODY(ASString,replace)
 {
-	ASString* th=static_cast<const ASString*>(obj);
+	ASString* th=static_cast<ASString*>(obj);
 	enum REPLACE_TYPE { STRING=0, FUNC };
 	REPLACE_TYPE type=(args[1]->getObjectType()==T_FUNCTION)?FUNC:STRING;
 	ASString* ret=Class<ASString>::getInstanceS(th->data);
@@ -3245,7 +3277,7 @@ bool lightspark::Boolean_concrete(ASObject* obj)
 	{
 		LOG(LOG_CALLS,_("Number to bool"));
 		double val=obj->toNumber();
-		if(val==0 || isnan(val))
+		if(val==0 || std::isnan(val))
 			return false;
 		else
 			return true;
@@ -3307,7 +3339,7 @@ ASFUNCTIONBODY(lightspark,isNaN)
 		return abstract_b(false);
 	else if(args[0]->getObjectType()==T_NUMBER)
 	{
-		if(isnan(args[0]->toNumber()))
+		if(std::isnan(args[0]->toNumber()))
 			return abstract_b(true);
 		else
 			return abstract_b(false);
@@ -3317,7 +3349,7 @@ ASFUNCTIONBODY(lightspark,isNaN)
 	else if(args[0]->getObjectType()==T_STRING)
 	{
 		double n=args[0]->toNumber();
-		return abstract_b(isnan(n));
+		return abstract_b(std::isnan(n));
 	}
 	else
 		throw UnsupportedException("Weird argument for isNaN");
